<?php

namespace App\Http\Controllers;

use App\AllItem;
use App\Flip;
use App\Inventory;
use App\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Redis;

class FlipController extends Controller
{
    public function createGame(Request $r)
    {
        $user = $r->user();
        $side = $r->get('side');
        $count = $r->get('count');
        $items = $r->get('items');

        if ($side <= 0 || $side > 2) {
            return [
                'success' => false,
                'message' => 'error'
            ];
        }

        if ($count <= 0 || $count > 5) {
            return [
                'success' => false,
                'message' => 'error'
            ];
        }

        $newItems = [];

        foreach ($items as $id => $i) {
            $item = Inventory::query()->with(['item'])->where('user_id', $user->id)->find($id);

            if (!$item) {
                return [
                    'success' => false,
                    'message' => 'item_not_found'
                ];
            }

            $newItems[] = $item;
        }

        if (count($newItems) < $count) {
            return [
                'success' => false,
                'message' => 'error'
            ];
        }

        foreach ($items as $id => $i) {
            Inventory::query()->with(['item'])->find($id)->delete();
        }

        $itemsGames = [];

        if ($count === 1) {
            $itemsGames = [$newItems];
        } else {
            $max = count($newItems) - $count + 1;
            $d = 0;
            for ($i = 0; $i < count($newItems); $i++) {
                if (isset($itemsGames[$d]) && count($itemsGames[$d]) >= $max) {
                    $d++;
                    $max = 1;
                }
                if (!isset($itemsGames[$d]) || count($itemsGames[$d]) < $max) {
                    $itemsGames[$d][] = $newItems[$i];
                }
            }
        }

        foreach ($itemsGames as $itemsGame) {
            $price = 0;
            $images = [];

            foreach ($itemsGame as $item) {
                $price += $item->item->price;
                $images[] = [
                    'img' => 'https://steamcommunity-a.akamaihd.net/economy/image/' . $item->item->image . '/128fx128f/image.png'
                ];
            }

            if ($side === 1) {
                $game = Flip::query()->create([
                    'blue_user_id' => $user->id,
                    'blue_user' => json_encode([
                        'avatar' => $user->avatar,
                        'username' => $user->username,
                        'uid' => $user->uid
                    ]),
                    'blue_weapons' => json_encode($images),
                    'price' => $price
                ]);
            } else {
                $game = Flip::query()->create([
                    'green_user_id' => $user->id,
                    'green_user' => json_encode([
                        'avatar' => $user->avatar,
                        'username' => $user->username,
                        'uid' => $user->uid
                    ]),
                    'green_weapons' => json_encode($images),
                    'price' => $price
                ]);
            }

            Redis::publish('flipNewGame', json_encode(Flip::query()->find($game->id)));
        }

        return [
            'success' => true,
            'message' => 'games_created'
        ];
    }

    public function joinGame(Request $r)
    {
        $user = $r->user();
        $items = $r->get('items');
        $id = $r->get('id');

        $game = Flip::query()->find($id);

        if (!$game || $game->status > 0) {
            return [
                'success' => false,
                'message' => 'game_not_found'
            ];
        }

        $newItems = [];
        $images = [];
        $price = 0;

        foreach ($items as $id => $i) {
            $item = Inventory::query()->with(['item'])->where('user_id', $user->id)->find($id);

            if (!$item) {
                return [
                    'success' => false,
                    'message' => 'item_not_found'
                ];
            }

            $newItems[] = $item;

            $images[] = [
                'img' => 'https://steamcommunity-a.akamaihd.net/economy/image/' . $item->item->image . '/128fx128f/image.png'
            ];

            $price += $item->item->price;
        }

        if ($price < $game->price) {
            return [
                'success' => false,
                'message' => 'not_money'
            ];
        }

        foreach ($items as $id => $i) {
            Inventory::query()->with(['item'])->find($id)->delete();
        }

        $ext = $price - $game->price;
        $user->increment('balance', $ext);

        if ($game->blue_user_id === null) {
            $game->update([
                'blue_user_id' => $user->id,
                'blue_user' => json_encode([
                    'avatar' => $user->avatar,
                    'username' => $user->username,
                    'uid' => $user->uid
                ]),
                'blue_weapons' => json_encode($images),
                'status' => 1
            ]);
        } else {
            $game->update([
                'green_user_id' => $user->id,
                'green_user' => json_encode([
                    'avatar' => $user->avatar,
                    'username' => $user->username,
                    'uid' => $user->uid
                ]),
                'green_weapons' => json_encode($images),
                'status' => 1
            ]);
        }

        $chance = mt_rand(1, 100);

        if ($chance <= 50) {
            $side = 'blue';
            $winner = $game->blue_user_id;
        } else {
            $side = 'green';
            $winner = $game->green_user_id;
        }

        $game->update([
            'winner' => $side,
            'winner_id' => $winner
        ]);

        $this->parseValues($game);

        Redis::publish('flipUpdateGame', json_encode($game));

        return [
            'success' => true,
            'message' => 'success',
            'balance' => $user->balance
        ];
    }

    public function getFlipGames()
    {
        $games = Flip::query()->where('status', '<', 4)->orderBy('id', 'desc')->get();

        foreach ($games as $game) {
            $this->parseValues($game);
        }

        return [
            'games' => $games,
            'allGames' => Flip::query()->where('status', 4)->orderBy('id', 'desc')->count('id')
        ];
    }

    public function flipSetStatus(Request $r)
    {
        $game = Flip::query()->find($r->get('id'));

        $game->update([
            'status' => $r->get('status')
        ]);

        if ($r->get('status') === 4) {
            $win = ($game->price * 2) - (($game->price * 2) * 0.03);
            $winner = User::query()->find($game->winner_id);

            $winItem = AllItem::query()->where('price', '<=', $win)->orderBy('price', 'DESC')->first();
            $extBalance = $win - $winItem->price;

            Inventory::query()->create([
                'user_id' => $winner->id,
                'item_id' => $winItem->id
            ]);

            if ($extBalance > 0) {
                $winner->increment('balance', $extBalance);
            }

            $game->update([
                'win_item_id' => $winItem->id
            ]);

            Redis::publish('updateInventories', json_encode([
                'id' => $winner->id,
                'balance' => $winner->balance
            ]));
        }
    }

    protected function parseValues($game)
    {
        if ($game->blue_user !== null) {
            $game->blue_user = json_decode($game->blue_user);
        }

        if ($game->blue_weapons !== null) {
            $game->blue_weapons = json_decode($game->blue_weapons);
        }

        if ($game->green_user !== null) {
            $game->green_user = json_decode($game->green_user);
        }

        if ($game->green_weapons !== null) {
            $game->green_weapons = json_decode($game->green_weapons);
        }

        return $game;
    }
}
