const app = require('express')(),
    server = require('http').createServer(options, app),
    Redis = require('redis'),
    RedisClient = Redis.createClient(),
    io = require('socket.io')(server),
    axios = require('axios');

const myArgs = process.argv.slice(2);
const domain = myArgs[0];
const SECRET_KEY = '9e,>6M>3qKV=Ar^fFF,cZN^/';

axios.defaults.baseURL = domain + '/api/bot/';

server.listen(8081);

RedisClient.subscribe('newMessage');
RedisClient.subscribe('loadChat');
RedisClient.subscribe('newFake');
RedisClient.subscribe('flipNewGame');
RedisClient.subscribe('flipUpdateGame');
RedisClient.subscribe('updateInventories');

RedisClient.on('message', async (channel, message) => {
    if (channel === 'newFake') {
        clearInterval(fakeMessages);
        if (message > 0.00) {
            fakeMessages = setInterval(() => {
                sendFakeMessage();
            }, 60000 * message);
        }
    } else if (channel === 'flipNewGame') {
        let game = JSON.parse(message);

        if (game.blue_user !== null) {
            game.blue_user = JSON.parse(game.blue_user);
        }

        if (game.blue_weapons !== null) {
            game.blue_weapons = JSON.parse(game.blue_weapons);
        }

        if (game.green_user !== null) {
            game.green_user = JSON.parse(game.green_user);
        }

        if (game.green_weapons !== null) {
            game.green_weapons = JSON.parse(game.green_weapons);
        }

        gamesFlip.unshift(game);

        io.sockets.emit(channel, game);
    } else if (channel === 'flipUpdateGame') {
        try {
            const game = JSON.parse(message);
            const gameID = gamesFlip.findIndex(x => x.id === game.id);

            if (typeof gameID === "undefined") {
                return;
            }

            gamesFlip[gamesFlip.findIndex(x => x.id === game.id)] = game;

            if (game.status === 1) {
                flipStartTimer(game.id);
            }
        } catch (e) {

        }
    } else {
        io.sockets.emit(channel, JSON.parse(message));
    }
});

io.on('connection', (socket) => {
    const updateOnline = () => {
        io.sockets.emit('online', Object.keys(io.sockets.adapter.rooms).length);
    };

    socket.on('disconnect', () => {
        updateOnline();
    });

    socket.on('newBet', (data) => {
        if (TIME <= 1) {
            return socket.emit('errorBet', 'game_started');
        }

        axios.post('newBet', {
            secretKey: SECRET_KEY,
            items: data.items,
            apiToken: data.apiToken,
            autoWithdraw: data.autoWithdraw
        })
            .then(res => {
                const result = res.data;

                if (result.success) {
                    if (typeof AUTO_WITHDRAWS[parseFloat(data.autoWithdraw).toFixed(2)] === "undefined") {
                        AUTO_WITHDRAWS[parseFloat(data.autoWithdraw).toFixed(2)] = [];
                    }

                    AUTO_WITHDRAWS[parseFloat(data.autoWithdraw).toFixed(2)].push({
                        token: data.apiToken
                    });

                    const index = bets.findIndex(item => {
                        return (result.bets.user.uid === item.user.uid)
                    });

                    if (index === -1) {
                        bets.push(result.bets);
                    } else {
                        bets.splice(index, 1, result.bets);
                    }

                    game = result.game;

                    io.sockets.emit('newBetsOne', result.bets);
                    io.sockets.emit('setStats', {
                        members: game.members,
                        skins: game.skins,
                        bank: game.bank
                    });

                    return socket.emit('successBet', {
                        price: result.price,
                        bet: result.bet
                    });
                } else {
                    return socket.emit('errorBet', result.message);
                }
            })
            .catch(err => {
                console.log(err);
                return socket.emit('errorBet', 'error');
            });
    });

    socket.on('getGame', (data) => {
        let arr = {
            status: 'timer',
            time: 0.00,
            bets: bets,
            graph: {
                translateA,
                translateB,
                a,
                b,
                c
            },
            multipliers: multipliers
        }

        if (_now > 0) {
            if (_now >= MULTIPLIER) {
                arr.type = 'explosion';
                arr.time = MULTIPLIER;
            } else {
                arr.type = 'multiplier';
                arr.time = _now;

                socket.emit('crashMultiplier', _now);
            }
        } else if (TIME > 0) {
            arr.type = 'timer';
            arr.time = parseFloat(TIME);

            socket.emit('crashTimer', parseFloat(TIME));
        } else if (_now >= MULTIPLIER) {
            arr.type = 'explosion';
            arr.time = MULTIPLIER;
        }

        socket.emit('setStats', {
            members: game.members,
            skins: game.skins,
            bank: game.bank
        });

        socket.emit('setGame', arr);
    });

    socket.on('take', (data) => {
        if (_now.toFixed(2) > MULTIPLIER) {
            return socket.emit('errorTake', 'end');
        }

        USER_WITHDRAWS.push(data.apiToken);

        axios.post('take', {
            secretKey: SECRET_KEY,
            apiToken: data.apiToken,
            multiplier: _now.toFixed(2)
        })
            .then(res => {
                const result = res.data;

                if (result.success) {
                    const index = bets.findIndex(item => {
                        return (result.bets.user.uid === item.user.uid)
                    });

                    if (index === -1) {
                        bets.push(result.bets);
                    } else {
                        bets.splice(index, 1, result.bets);
                    }

                    io.sockets.emit('newBetsOne', result.bets);
                    return socket.emit('successTake', {
                        win: result.win,
                        item: result.item,
                        newBalance: result.newBalance
                    });
                } else {
                    return socket.emit('errorTake', result.message);
                }
            })
            .catch(err => {
                console.log(err)
                return socket.emit('errorTake', 'error');
            })
    });

    socket.on('getFlipGames', () => {
        socket.emit('setFlipGames', gamesFlip);
        socket.emit('setAllGamesFlip', allGamesFlip);
    });

    updateOnline();
});

let TIME = 0,
    AUTO_WITHDRAWS = [],
    WITHDRAWS_MULTIPLIER = [],
    MULTIPLIER = 0.00,
    USER_WITHDRAWS = [],
    _now = 0,
    _i = 0,
    game = {},
    bets = {},
    fakeMessages = null,
    gamesFlip = [],
    allGamesFlip = 0,
    translateA = 2,
    translateB = 0,
    a = 0,
    b = 0,
    c = 0,
    multipliers = [1.00, 2.00, 3.00, 4.00];

const startTimer = () => {
    TIME = 9.00;

    setTimeout(() => {
        fakeBets();
    }, 1000 * randomInteger(1, 4));

    const TIME_TO_START = setInterval(() => {
        if (parseFloat(TIME) === 1.00) {
            setStatus(1);
        }
        if (parseFloat(TIME) <= 0.00) {
            clearInterval(TIME_TO_START);
            startGame();
            return;
        }

        TIME = parseFloat(parseFloat(TIME) - 0.1).toFixed(2);

        io.sockets.emit('crashTimer', parseFloat(TIME).toFixed(2));
    }, 100);
};

const startGame = () => {
    let float = MULTIPLIER;
    let _now_old = -1;

    _i = 0,
        _now = 0,
        translateA = 1,
        translateB = 0,
        a = 0,
        b = 0,
        c = 0;

    io.sockets.emit('stopTimer', 1);

    const START_GAME_INTERVAL = setInterval(async () => {
        _i++;
        _now = parseFloat(Math.pow(Math.E, 0.00006 * _i * 1000 / 20));

        if (_now_old.toFixed(2).toString() !== _now.toFixed(2).toString()) {
            _now_old = _now;
            if (parseFloat(_now.toFixed(2)) >= float) {
                clearInterval(START_GAME_INTERVAL);
                io.sockets.emit('crashCrashed', MULTIPLIER);
                crashBets();
            } else {
                for (const [key, value] of Object.entries(AUTO_WITHDRAWS)) {
                    if (key <= parseFloat(_now.toFixed(2))) {
                        if (typeof AUTO_WITHDRAWS[key] !== "undefined") {
                            withdrawAuto(key, AUTO_WITHDRAWS[key]);
                        }
                    }
                }

                if (translateA < 174.188) {
                    translateA += 0.57396;
                }

                if (translateB < 174.188) {
                    translateB += 0.58062666666;
                }

                if (a < 55.016) {
                    a += 0.18438666666;
                }

                if (b < 128.370666667) {
                    b += 0.43790222222;
                }

                if (c < 181.393333333) {
                    c += 0.61464444444;
                }

                if (parseFloat(_now.toFixed(2)) > parseFloat(parseFloat(multipliers[3]).toFixed(2))) {
                    multipliers.shift();
                    multipliers.push(parseFloat(_now.toFixed(0)) + 3);
                }

                io.sockets.emit('crashMultiplier', {
                    multiplier: _now,
                    translateA,
                    translateB,
                    a,
                    b,
                    c,
                    multipliers
                });
            }
        }
    }, 50)
};

const getGame = () => {
    axios.post('getGame', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            game = res.data.game;
            bets = res.data.bets;

            io.sockets.emit('clearBets', res.data.history);

            WITHDRAWS_MULTIPLIER = [];
            AUTO_WITHDRAWS = [];
            USER_WITHDRAWS = [];

            multipliers = [1.00, 2.00, 3.00, 4.00];

            if (game.status === 0) {
                startTimer();
            }

            if (game.status === 1) {
                startGame();
            }
        })
        .catch(err => {
            console.log(err);
            setTimeout(() => {
                getGame();
            }, 2000);
        });
};

const setStatus = (status) => {
    axios.post('setStatus', {
        secretKey: SECRET_KEY,
        status: status
    })
        .then(res => {
            if (status === 1) {
                MULTIPLIER = res.data;
            }
        })
        .catch(err => {
            console.log(err);
        });
};

const crashBets = () => {
    axios.post('crashBets', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            setStatus(2);
            bets = res.data;
            io.sockets.emit('newBetsCrash', res.data);

            setTimeout(() => {
                _i = 0, _now = 0;
                getGame();
            }, 3000);
        })
        .catch(err => {
            console.log(err);
        });
};

const withdrawAuto = (multiplier, members) => {
    if (typeof WITHDRAWS_MULTIPLIER[multiplier] === "undefined") {
        WITHDRAWS_MULTIPLIER[multiplier] = 1;

        axios.post('autoTake', {
            secretKey: SECRET_KEY,
            members: members,
            multiplier: multiplier,
            autoWithdraws: USER_WITHDRAWS
        })
            .then(res => {
                const result = res.data;

                bets = result.bets;
                io.sockets.emit('newBets', bets);

                result.members.forEach(member => {
                    io.sockets.emit('successTake', {
                        win: member.win,
                        item: member.item,
                        newBalance: member.newBalance,
                        user_id: member.user_id
                    });
                });
            })
    }
};

const getWithdraws = () => {
    axios.post('getWithdraws', {
        secretKey: SECRET_KEY
    })
        .then(res => {

        })
        .catch(err => {
            console.log(err);
        });
};

const fakeBets = () => {
    axios.post('fakeBets', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            const result = res.data;

            result.members.forEach(withdraw => {
                if (typeof AUTO_WITHDRAWS[parseFloat(withdraw.autoWithdraw).toFixed(2)] === "undefined") {
                    AUTO_WITHDRAWS[parseFloat(withdraw.autoWithdraw).toFixed(2)] = [];
                }

                AUTO_WITHDRAWS[parseFloat(withdraw.autoWithdraw).toFixed(2)].push({
                    token: withdraw.apiToken
                });
            });

            bets = result.bets;
            game = result.game;

            io.sockets.emit('newBets', bets);
            io.sockets.emit('setStats', {
                members: game.members,
                skins: game.skins,
                bank: game.bank
            });
        })
        .catch(err => {
            console.log(err);
        });
};

const randomInteger = (min, max) => {
    let rand = min + Math.random() * (max + 1 - min);
    return Math.floor(rand);
}

const getLoadFakeMessages = () => {
    axios.post('getLoadFakeMessages', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            const timer = res.data;

            if (timer > 0.00) {
                fakeMessages = setInterval(() => {
                    sendFakeMessage();
                }, 60000 * timer);
            }
        })
        .catch(err => {
        	console.log("ERROR");
            console.log(err);
        });
};

const sendFakeMessage = () => {
    axios.post('sendFakeMessage', {
        secretKey: SECRET_KEY
    })
        .then(res => {

        })
        .catch(err => {
            console.log(err);
        });
};

const getRaffle = () => {
    axios.post('getRaffle', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            io.sockets.emit('setRaffle', res.data);
        })
        .catch(err => {
            console.log(err);
        });
};

const getFlipGames = () => {
    axios.post('getFlipGames', {
        secretKey: SECRET_KEY
    })
        .then(res => {
            gamesFlip = res.data.games;
            allGamesFlip = res.data.allGames;
            gamesFlip.forEach(game => {
                if (game.status === 1) {
                    flipStartTimer(game.id);
                }
            });
        })
        .catch(err => {
            setTimeout(() => {
                getFlipGames();
            }, 3000);
        });
};

const flipStartTimer = (gameID) => {
    let timer = 3;

    try {
        gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].countdown = timer;
        io.sockets.emit('flipUpdateGame', gamesFlip[gamesFlip.findIndex(x => x.id === gameID)]);

        const timerToStart = setInterval(() => {
            gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].countdown = timer;

            if (timer === 0) {
                clearInterval(timerToStart);
                flipSetStatus(gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].id, 2);
                flipStartGame(gameID);
            }

            timer--;
        }, 1000);
    } catch (e) {
        console.log(`Ошибка flipStartTimer ${gameID} - ${e}`);
    }
};

const flipStartGame = (gameID) => {
    try {
        gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].status = 2;

        let finishTime = 5.00;

        gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].finishTime = finishTime;
        io.sockets.emit('flipUpdateGame', gamesFlip[gamesFlip.findIndex(x => x.id === gameID)]);

        const timerFinish = setInterval(() => {
            gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].finishTime = finishTime;

            if (finishTime <= 0.00) {
                clearInterval(timerFinish);

                flipSetStatus(gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].id, 3);
                gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].status = 3;
                gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].win = (parseFloat(gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].price * 2) - parseFloat((gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].price * 2) * 0.03)).toFixed(2);
                io.sockets.emit('flipUpdateGame', gamesFlip[gamesFlip.findIndex(x => x.id === gameID)]);

                setTimeout(() => {
                    flipSetStatus(gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].id, 4);
                    gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].status = 4;
                    io.sockets.emit('flipUpdateGame', gamesFlip[gamesFlip.findIndex(x => x.id === gameID)]);

                    setTimeout(() => {
                        gamesFlip[gamesFlip.findIndex(x => x.id === gameID)].status = 5;

                        io.sockets.emit('flipUpdateGame', gamesFlip[gamesFlip.findIndex(x => x.id === gameID)]);
                        gamesFlip.splice(gamesFlip.findIndex(x => x.id === gameID), 1);

                        allGamesFlip++;
                        io.sockets.emit('setAllGamesFlip', allGamesFlip);
                    }, 10000);
                }, 4000);
            }

            finishTime = parseFloat(finishTime - 0.1).toFixed(1);
        }, 100);
    } catch (e) {
        console.log(`Ошибка flipStartGame ${gameID} - ${e}`);
    }
};

const flipSetStatus = (id, status) => {
    axios.post('flipSetStatus', {
        id: id,
        status: status,
        secretKey: SECRET_KEY
    })
        .then(res => {

        })
        .catch(err => {
            setTimeout(() => {
                flipSetStatus(id, status);
            }, 3000);
        });
};

getGame();
getLoadFakeMessages();
getFlipGames();

setInterval(() => {
    getWithdraws();
}, 15000);

setInterval(() => {
    getRaffle();
}, 60000);
